<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\ResponseHelper;
use App\Model\RechargeApi;
use JsonSerializable;
use Illuminate\Support\Facades\Auth;
use Log;

class OrderCancelAndRefundController extends Controller {

    private $helper;
    private $rechargeObj;

    public function __construct() {

        $this->helper = new ResponseHelper();
        $this->rechargeObj = new RechargeApi();
    }

    /**
     * Get Shopify order details
     */
    public function  cancelAndRefundOrder(Request $request) {

        $orderId = !empty($request->get('id')) ? $request->get('id') : '';
        $responseData = [];

        if (empty($orderId)) {

            $errorMessage = 'Failed to get the order details from Shopify';
            $error = "Failed to get the orderId from the URL";
            $responseData = [
                "error" => $errorMessage
            ];
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            return view('order',compact('responseData'));
        }

        $responseData = [
            "orderId" => $orderId
        ];
        return view('orderCancel',compact('responseData'));
    }

    /**
     * Cancel and refund Shopify order and refund recharge charge
     */
    public function processOrderCancelAndRefund(Request $request) {

        $requestData = $request->input('data');
        $requestDecoded = json_decode($requestData);
        $errorMessage = '';
        $error = '';
        $rechargeSkippedApiErrors = ["charge is already fully refunded"];
        $orderId = !empty($requestDecoded->orderId) ? trim($requestDecoded->orderId) : '';
        $checkboxCheckedStatus = !empty($requestDecoded->subscriptionCancelStatus) ? trim($requestDecoded->subscriptionCancelStatus) : '';

        if (empty($orderId)) {

            $errorMessage = 'Not found the order details, please try again';
            $error = "Required parameter orderId is empty in the request, Request:".$requestData;
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }

        $shop = Auth::user();
        $orderDetails = $shop->api()->rest('GET', htmlentities('/admin/orders/'.$orderId.'.json'));

        $orderDetails = isset($orderDetails['body']['order']) ? 
        $orderDetails['body']['container']['order'] : '';

        if (!empty($orderDetails['error']) || !empty($orderDetails['errors']) || empty($orderDetails)) {

            $shError  = "Unknown error";
            if (!empty($orderDetails['error'])) {

                $shError = json_encode($orderDetails['error']);
            } else if (!empty($orderDetails['errors'])) {

                $shError = json_encode($orderDetails['errors']);
            } else if (empty($orderDetails)) {

                $shError = "Failed to get the order details from Shopify";
            }

            $errorMessage = 'Failed to get the order details from Shopify';
            $error = "Failed to get the order details from Shopify - OrderId: ".$orderId.", Error: ".$shError;
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }

        $gifteeEmailAddressStatus = 0;
        if (!empty($orderDetails['line_items'][0]['properties'])) {

            foreach ($orderDetails['line_items'][0]['properties'] as $propKey => $propValue) {

                if (strtolower(trim($propValue['name'])) == 'giftee_email') {

                    if (!empty($propValue['value'])) {

                        $gifteeEmailAddressStatus = 1;
                    }
                }
            }
        }

        $lineItemId = !empty($orderDetails['line_items'][0]['id']) ?
        $orderDetails['line_items'][0]['id'] : '';
        $productQuantity = !empty($orderDetails['line_items'][0]['quantity']) ?
        $orderDetails['line_items'][0]['quantity'] : '';
        $productVariantId = !empty($orderDetails['line_items'][0]['variant_id']) ?
        $orderDetails['line_items'][0]['variant_id'] : '';
        $orderCancelledDate = !empty($orderDetails['cancelled_at']) ?
        trim($orderDetails['cancelled_at']) : '';
        $financialStatus = !empty($orderDetails['financial_status']) ?
        trim($orderDetails['financial_status']) : '';
        $fulfillmentStatus = !empty($orderDetails['fulfillment_status']) ? 
        trim($orderDetails['fulfillment_status']) : '';

        if ($gifteeEmailAddressStatus == 1) {

            $errorMessage = 'Please refund gift subscriptions directly through ReCharge';
            $error = "Skipped: The order is a gift order(OrderId:".$orderId.")";
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }

        if (!empty($fulfillmentStatus)) {

            if (strtolower(trim($fulfillmentStatus)) == 'fulfilled') {

                $errorMessage = 'The order is fulfilled';
                $error = "The order is fulfilled(OrderId:".$orderId.")";
                Log::channel('customOrderCancelAndRefundLog')->info($error);
                $this->returnData("600", $errorMessage);
            }
        }

        if (!empty($orderCancelledDate) && strtolower($financialStatus) == 'refunded') {

            $errorMessage = 'The order is already refunded and cancelled from Shopify';
            $error = "The order is already refunded and cancelled from Shopify(OrderId:".$orderId.")";
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }

        // Get charge details from recharge by shopify order ID
        $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($orderId);
        if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
            empty($chargeDetails) || empty($chargeDetails->charges)) {

            $rechargeError  = "Unknown error";
            if (!empty($chargeDetails->error)) {

                $rechargeError = json_encode($chargeDetails->error);
            } else if (!empty($chargeDetails->errors)) {

                $rechargeError = json_encode($chargeDetails->errors);
            } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError = 'Failed to get the charge details by using the given orderId';
            }

            $errorMessage = 'Failed to get the order details from recharge';
            $error = "Failed to get the charge details from the recharge(OrderId:".$orderId."), Error: (".$rechargeError.")";
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }

        $chargeId = $chargeDetails->charges[0]->id;
        $subscriptionId = $chargeDetails->charges[0]->line_items[0]->subscription_id;
        $rechargeChargeAmount = $chargeDetails->charges[0]->total_price;
        $rechargeHashToken = trim($chargeDetails->charges[0]->customer_hash);

        // Get shopify order transactional details
        $transactionalDetails = $shop->api()->rest('GET', htmlentities('/admin/orders/'.$orderId.'/transactions.json'));

        $transactionalDetails = isset($transactionalDetails['body']['transactions']) ? 
        $transactionalDetails['body']['container']['transactions'] : '';

        if (!empty($transactionalDetails['error']) || !empty($transactionalDetails['errors']) ||
            empty($transactionalDetails)) {

            $shTransactionalError  = "Unknown error";
            if (!empty($transactionalDetails['error'])) {

                $shTransactionalError = json_encode($transactionalDetails['error']);
            } else if (!empty($transactionalDetails['errors'])) {

                $shTransactionalError = json_encode($transactionalDetails['errors']);
            } else if (empty($transactionalDetails) || empty($transactionalDetails['transactions'])) {

                $shTransactionalError = 'Failed to get the shopify transaction details by using the given 
                orderId';
            }

            $errorMessage = 'Failed to get the transactional details from Shopify';
            $error = "Failed to get the Shopify order transactional details(OrderId:".$orderId."), Error: (".$shTransactionalError.")";
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }
        $transactionalId = '';
        $transactionalGateWay = '';
        foreach ($transactionalDetails as $transKey => $transValue) {

            if ($transValue['kind'] == 'sale') {
                $transactionalId = $transValue['id'];
                $transactionalGateWay = $transValue['gateway'];
            }
        }

        // Refund shopify order via shopify API
        $shopifyRefundRequest = [];
        $shopifyRefundRequest = array(
            "refund" => array(
                "currency" => "USD",
                "notify" => true,
                "shipping" => array(
                    "full_refund" => true
                ),
                "refund_line_items" => array(
                    array(
                        "line_item_id" => $lineItemId,
                        "quantity" => $productQuantity,
                        "line_item" => array(
                            "variant_id" => $productVariantId
                        )
                    )
                ),
                "transactions" => array(
                    array(
                        "parent_id" => $transactionalId,
                        "amount" => $rechargeChargeAmount,
                        "kind" => "refund",
                        "gateway" => $transactionalGateWay
                    )
                )
            )
        );

        // Refund Shopify order
        if ($financialStatus != 'refunded') {
            $refundResponse = $shop->api()->rest('POST', htmlentities('/admin/orders/'.$orderId.'/refunds.json'), $shopifyRefundRequest);

            $refundResponse = isset($refundResponse['body']['refund']) ? 
            $refundResponse['body']['container']['refund'] : '';

            if (!empty($refundResponse['error']) || !empty($refundResponse['errors']) ||
                empty($refundResponse)) {

                $shopifyRefundError  = "Unknown error";
                if (!empty($refundResponse['error'])) {

                    $shopifyRefundError = json_encode($refundResponse['error']);
                } else if (!empty($refundResponse['errors'])) {

                    $shopifyRefundError = json_encode($refundResponse['errors']);
                } else if (empty($refundResponse)) {

                    $shopifyRefundError = 'The refunded response from Shopify is empty';
                }

                $errorMessage = 'Failed to refund the order from Shopify';
                $error = "Failed to refund the order from Shopify(OrderId:".$orderId."), Error: (".$shopifyRefundError.")";
                Log::channel('customOrderCancelAndRefundLog')->info($error);
                $this->returnData("600", $errorMessage);
            }
        }

        // Refund recharge charge
        $refundAmountRequest = array(
            "amount" => $rechargeChargeAmount
        );
        $chargeRefundedDetails = $this->rechargeObj->refundRechargeChargeById($chargeId, json_encode($refundAmountRequest));
        $alreadyRefundedStatus = 0;
        if (!empty($chargeRefundedDetails->error)) {
            $refundErrorMsg = trim($chargeRefundedDetails->error);
            if (in_array($refundErrorMsg, $rechargeSkippedApiErrors)) {
                $alreadyRefundedStatus = 1;
            }
        }
        if ((!empty($chargeRefundedDetails->error) || !empty($chargeRefundedDetails->errors) ||
            empty($chargeRefundedDetails)) && $alreadyRefundedStatus == 0) {

            $rechargeRefundError  = "Unknown error";
            if (!empty($chargeRefundedDetails->error)) {

                $rechargeRefundError = json_encode($chargeRefundedDetails->error);
            } else if (!empty($chargeRefundedDetails->errors)) {

                $rechargeRefundError = json_encode($chargeRefundedDetails->errors);
            } else if (empty($chargeRefundedDetails)) {

                $rechargeRefundError = 'The refunded response from recharge is empty';
            }

            $errorMessage = 'Failed to refund the order from recharge';
            $error = "Failed to refund the charge from recharge(OrderId:".$orderId."), Error: (".$rechargeRefundError.")";
            Log::channel('customOrderCancelAndRefundLog')->info($error);
            $this->returnData("600", $errorMessage);
        }

        // Cancel shopify order
        if (empty($orderCancelledDate)) {
            $orderCancelledDetails = $shop->api()->rest('POST', htmlentities('/admin/orders/'.$orderId.'/cancel.json'));

            $orderCancelledDetails = isset($orderCancelledDetails['body']['order']) ? 
            $orderCancelledDetails['body']['container']['order'] : '';

            if (!empty($orderCancelledDetails['error']) || !empty($orderCancelledDetails['errors']) ||
                empty($orderCancelledDetails)) {

                $orderCancelError  = "Unknown error";
                if (!empty($orderCancelledDetails['error'])) {

                    $orderCancelError = json_encode($orderCancelledDetails['error']);
                } else if (!empty($orderCancelledDetails['errors'])) {

                    $orderCancelError = json_encode($orderCancelledDetails['errors']);
                } else if (empty($orderCancelledDetails)) {

                    $orderCancelError = 'The order canceled response from Shopify is empty';
                }

                $errorMessage = 'Failed to cancel the order from Shopify';
                $error = "Failed to cancel the order from Shopify(OrderId:".$orderId."), Error: (".$orderCancelError.")";
                Log::channel('customOrderCancelAndRefundLog')->info($error);
                $this->returnData("600", $errorMessage);
            }
        }

        if ($checkboxCheckedStatus == 'true') {

            // Cancel recharge subscription
            $cancelRequest = array(
                "cancellation_reason" => "other reason"
            );
            $subscriptionCancelledDetails = $this->rechargeObj->cancelRechargeSubscription($subscriptionId, 
            json_encode($cancelRequest));
            if (!empty($subscriptionCancelledDetails->error) || !empty($subscriptionCancelledDetails->errors) ||
                empty($subscriptionCancelledDetails)) {

                $subscriptionCancelError  = "Unknown error";
                if (!empty($subscriptionCancelledDetails->error)) {

                    $subscriptionCancelError = json_encode($subscriptionCancelledDetails->error);
                } else if (!empty($subscriptionCancelledDetails->errors)) {

                    $subscriptionCancelError = json_encode($subscriptionCancelledDetails->errors);
                } else if (empty($subscriptionCancelledDetails)) {

                    $subscriptionCancelError = 'The subscription cancelled response from Shopify is empty';
                }

                $errorMessage = 'Failed to cancel the subscription from Recharge';
                $error = "Failed to cancel the subscription from Recharge(SubscriptionId:".$subscriptionId."), Error: (".$subscriptionCancelError.")";
                Log::channel('customOrderCancelAndRefundLog')->info($error);
                $this->returnData("600", $errorMessage);
            }
        }
        Log::channel('customOrderCancelAndRefundLog')->info("Successfully processed the order cancel request, OrderId:".$orderId);
        $this->returnData("200", $errorMessage);
    }

    /**
     * Response function
     */
    public function returnData($status, $error) {

        $returnData = array(
            'status' => $status,
            'error' => $error
        );
        echo json_encode($returnData);
        die();
    }


}
