<?php
namespace App\Model;
use Illuminate\Database\Eloquent\Model;

class RechargeApi extends Model {

    private $rechargeUrl;
    private $accessToken;
    private $rechargeObj;
    private $rechargeApiKey;

    function __construct() {

        $this->rechargeUrl = env('RECHARGE_URL', true);
        $this->rechargeApiKey = env('RECHARGE_API_KEY',true);
    }
   
   
    public function getChargeById($id){

        $method = 'GET';
        $path = $this->rechargeUrl . 'charges/'.$id;
        $response = $this->call($method, $path);
        return json_decode($response);

    }

    /**
     * update subscription by ID
     * @param $subscription_id
     * @param array() $data
     * @return JsonSerializable
     */

    function updateSubscription($data = NULL) {

        $method = 'GET';
        //$data = ["data" => $data];
        $path = env('UPDATE_SUB_ENDPOINT_URL', true).'/updateSubscriptionLineItemProperties';
        $response = $this->callEndpoint($method, $path, $data);
        $response = trim($response,")");
        $response = ltrim($response,"update(");
        return json_decode($response,true);

    }

    /**
     * get subscription by subscription_id
     * @param $subscription_id
     * @return JsonSerializable
     */

    function getSubscription($subscription_id) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id;
        $response = $this->call($method, $path);
        return json_decode($response);

    }

    function getOrderBySopifyOrderId($order_id) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/orders?shopify_order_id=' . $order_id;
        $response = $this->call($method, $path);
        return json_decode($response);

    }

    function activateSubscription($subscription_id) {

        $method = 'POST';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id . '/activate';
        $data="{}";
        $response = $this->call($method, $path, $data);
        return json_decode($response);

    }

    function createCheckout($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/checkouts';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    function updateCheckout($token, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    function retrieveCheckout($token, $status = 'checkout') {

        $method = 'GET';
        if($status == 'checkout') {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
            $token; 
        } else {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
                $token.'/shipping_rates';
        }
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    function updateShippingLine($token, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    function getRechargeCustomerDetailsByID($customerID) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers/'.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    function getRechargeCustomerDetailsByEmail($email) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers?email='.$email;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    function getSubscriptionByCustomerID($customerID) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Get charge details by shopify order ID
     */
    function getChargeDetailsByShopifyOrderId($shopifyOrderId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?shopify_order_id='.$shopifyOrderId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Refund recharge charge by ID
     */
    function refundRechargeChargeById($chargeId, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/charges/'.$chargeId.'/refund';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Cancel recharge subscription by ID
     */
    function cancelRechargeSubscription($subscriptionId, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscriptionId.'/cancel';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    function call($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . env('RECHARGE_API_KEY',true),
            "content-type: application/json",
        ));
     
        $response = curl_exec($curl);
        return $response;

    }
}
